/** 
 * @file stk11xx-bayer.c
 * @author Martin ROOS
 * @date 2006-01-14
 * @version v0.0.1
 *
 * @brief Driver for Syntek USB video camera
 *
 * @note Copyright (C) Nicolas VIVIEN
 *
 * @par Licences
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 * @par SubVersion
 *   $Date: $
 *   $Revision: $
 *   $Author: $
 *   $HeadURL: $
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/slab.h>
#include <linux/kref.h>

#include <linux/usb.h>
#include <linux/videodev2.h>
#include <media/v4l2-common.h>

#include "stk11xx.h"


/**
 * @def STK11XX_AVG2
 *   Calculate the adverage value from the values x and y
 *
 * @def STK11XX_AVG4
 *   Calculate the adverage value from the values a, b, c and d
 */
#define STK11XX_AVG2(x,y) (uint8_t) ( ((int)x + (int)y) / 2)
#define STK11XX_AVG4(a,b,c,d) (uint8_t) ( ((int)a + (int)b + (int)c + (int)d) / 4)



/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief Decompress a frame
 *
 * This function permits to decompress a frame from the video stream.
 */
int stk11xx_decompress(struct usb_stk11xx *dev)
{
	int width;
	int height;
	int factor;

	void *data;
	void *image;
	void *decompress;
	struct stk11xx_frame_buf *framebuf;

	if (dev == NULL)
		return -EFAULT;

	framebuf = dev->read_frame;
	decompress = dev->decompress_data;

	if (framebuf == NULL)
		return -EFAULT;

	image  = dev->image_data;
	image += dev->images[dev->fill_image].offset;

	data = framebuf->data;

	switch (dev->resolution) {
		case STK11XX_80x60:
			factor = 8;
			width = stk11xx_image_sizes[STK11XX_640x480].x;
			height = stk11xx_image_sizes[STK11XX_640x480].y;
			break;

		case STK11XX_160x120:
			factor = 4;
			width = stk11xx_image_sizes[STK11XX_640x480].x;
			height = stk11xx_image_sizes[STK11XX_640x480].y;
			break;

		case STK11XX_320x240:
			factor = 2;
			width = stk11xx_image_sizes[STK11XX_640x480].x;
			height = stk11xx_image_sizes[STK11XX_640x480].y;
			break;

		case STK11XX_640x480:
			factor = 1;
			width = stk11xx_image_sizes[STK11XX_640x480].x;
			height = stk11xx_image_sizes[STK11XX_640x480].y;
			break;

		case STK11XX_800x600:
			factor = 1;
			width = stk11xx_image_sizes[STK11XX_1280x1024].x;
			height = stk11xx_image_sizes[STK11XX_1280x1024].y;
			break;

		case STK11XX_1024x758:
			factor = 1;
			width = stk11xx_image_sizes[STK11XX_1280x1024].x;
			height = stk11xx_image_sizes[STK11XX_1280x1024].y;
			break;

		case STK11XX_1280x1024:
			factor = 1;
			width = stk11xx_image_sizes[STK11XX_1280x1024].x;
			height = stk11xx_image_sizes[STK11XX_1280x1024].y;
			break;

		default:
			return -EFAULT;
	}

	stk11xx_bayer_to_rgb(decompress, data, width, height);

	stk11xx_resize_image(image, decompress, width, height, factor);

	stk11xx_correct_brightness(image, width/factor, height/factor, dev->vsettings.brightness);


	return 0;
}




/** 
 * @param bayer Buffer to Bayer data
 * @param width Width of frame
 * @param height Height of frame
 *
 * @retval rgb Buffer to RGB data
 *
 * @brief Convert a data from Bayer to RGB
 *
 * This function permits to convert a data from bayer to RGB space.
 */
void stk11xx_bayer_to_rgb(uint8_t *rgb, const uint8_t *bayer,
		const int width, const int height) {
	// some initial variables
	int i,j;
	int x,y;

	// variables for interpolation steps
	int rpos; // current read pos
	int wpos; // current write pos
	int ltpos; // line on top of us pos
	int lbpos; // line below of us pos

	//  variables for border interpolation
	int runlength = width > height ? width : height;
	int lastrow = ( (height-1)*width ) * 3; 
	int lastcolumn = (width - 1) * 3;


	// blit out initial data
	for (j=0, y=0; y<height; y++) {
		for (i=0, x=0; x<width; x++) {
			if (y&1) {
				// GBGBGB line 
				if (x&1) { // blue
					rgb[j * width * 3 + i*3 + 2] = bayer[(height-y) * width + x];	
				}
				else { // green
					rgb[j * width * 3 + i*3 + 1] = bayer[(height-y) * width + x];
				}
			} 
			else {
				// RGRGRG line 
				if (x&1) { // green
					rgb[j * width * 3 + i*3 + 1] = bayer[(height-y) * width + x];	
				}
				else { // red
					rgb[j * width * 3 + i*3 + 0] = bayer[(height-y) * width + x];
				}	
			}

			i++;
		}

		j++;
	}

	


	// blit in everything but the borders
	for(y = 1; y < (height - 1); y++) {
		if( y&1 ) {
			// GBGBGB line 
			for( x = 1; x < (width - 1); x++) {
				rpos = (height-y)*width + x;
				wpos = y*width + x;
				ltpos = rpos - width;
				lbpos = rpos + width;
				
				if( x&1 ) { // blue is known
					// calculate red
					rgb[(wpos)*3 + 0] = STK11XX_AVG4( \
						/*Rtl*/ bayer[ltpos - 1], \
						/*Rtr*/ bayer[ltpos +1], \
						/*Rbl*/ bayer[lbpos -1], \
						/*Rbr*/ bayer[lbpos +1] \
					);
					// calculate green
					rgb[(wpos)*3 + 1] = STK11XX_AVG4( \
						/*Gt*/ bayer[ltpos], \
						/*Gl*/ bayer[rpos-1], \
						/*Gb*/ bayer[lbpos], \
						/*Gr*/ bayer[rpos+1] \
					);
	
				} else { // green is known
					// calculate red 
					rgb[(wpos)*3 + 0] = STK11XX_AVG2( \
						/*Rt*/ bayer[ltpos], \
						/*Rb*/ bayer[lbpos] \
					);
					// calculate blue
					rgb[(wpos)*3 + 2] = STK11XX_AVG2( \
						/*Bl*/ bayer[rpos-1], \
						/*Br*/ bayer[rpos+1] \
					);
				}
			}
		} else {
			// RGRG line 
			for( x = 1; x < (width - 1); x++) {
				rpos = (height-y)*width + x;
				wpos = y*width + x;
				ltpos = rpos - width;
				lbpos = rpos + width;
	
				if( x&1 ) { // green is known
					// calculate red 
					rgb[(wpos)*3 + 0] = STK11XX_AVG2( \
						/*Rl*/ bayer[rpos-1], \
						/*Rr*/ bayer[rpos+1] \
					);
					// calculate blue
					rgb[(wpos)*3 + 2] = STK11XX_AVG2( \
						/*Bt*/ bayer[ltpos], \
						/*Bb*/ bayer[lbpos] \
					);
	
				} else { // red is known
					// calculate blue
					rgb[(wpos)*3 + 2] = STK11XX_AVG4( \
						/*Btl*/ bayer[ltpos - 1], \
						/*Btr*/ bayer[ltpos + 1], \
						/*Bbl*/ bayer[lbpos - 1], \
						/*Bbr*/ bayer[lbpos + 1] \
					);
					// calculate green
					rgb[(wpos)*3 + 1] = STK11XX_AVG4( \
						/*Gt*/ bayer[ltpos], \
						/*Gl*/ bayer[rpos-1], \
						/*Gb*/ bayer[lbpos], \
						/*Gr*/ bayer[rpos+1] \
					);
				}
			}		
		}
	}


	for(i=0;i<runlength;i++) {
		if(i < width) {
			//== top border: we have certain red and green pixels  ==
			if( (i & 1) == 1) {
				// red , we use avg4(bottom, bottom, leftbottom, left) 
				rgb[i*3] = STK11XX_AVG4( \
					rgb[(i+width)*3], \
					rgb[(i+width)*3], \
					rgb[(i+width-1)*3], \
					rgb[(i-1)*3] \
				);
			} else  { // if( (i & 1) == 0) 
				// green , we use avg4(bottom, bottom, rightbottom, right) 
				rgb[i*3+1] = STK11XX_AVG4( \
					rgb[(i+width)*3 + 1], \
					rgb[(i+width)*3 + 1], \
					rgb[(i+width+1)*3 + 1], \
					rgb[(i+1)*3 + 1] \
				);
			}
			// blue has to be calculated for every pixel, we'll use bottom and one more bottom
			rgb[i*3+2] = STK11XX_AVG2( \
				rgb[(i+width)*3 + 2], \
				rgb[(i+width*2)*3 + 2] \
			);

			//== bottom border : we have green and blue pixels for sure ==
			if( (i & 1) == 1) {
				// green , we use avg4(top, top, lefttop, left) 
				rgb[ lastrow + i*3 + 1] = STK11XX_AVG4( \
						rgb[lastrow + (i-width)*3 + 1], \
						rgb[lastrow + (i-width)*3 + 1], \
						rgb[lastrow + (i-width-1)*3 + 1], \
						rgb[lastrow + (i-1)*3 + 1] \
				);
			} else { // if( (i & 1) == 0 ) 
				// blue , we use avg4(top, top, righttop, right)
				rgb[lastrow + i*3 + 2] = STK11XX_AVG4( \
						rgb[lastrow +(i-width)*3 + 2], \
						rgb[lastrow + (i-width)*3 + 2], \
						rgb[lastrow + (i-width+1)*3 + 2], \
						rgb[lastrow + (i+1)*3 + 2] \
				);
			}
			// red has to be calculated for every pixel, we'll use top and another top 
			rgb[lastrow + i*3] = STK11XX_AVG2( \
						rgb[lastrow + (i-width)*3], \
						rgb[lastrow + (i-width*2)*3] \
			);
				
			
		}
		if(i < height) { // the left and right borders
		
			//left border
			if( (i&1) == 0) {
				// calc green , just avg right and bottom
				rgb[(i*width)*3+1] = STK11XX_AVG2( \
						rgb[(i*width+1)*3 + 1], \
						rgb[((i+1)*width)*3 + 1] \
				);
				// calc blue , avg right and right bottom 
				rgb[(i*width)*3+2] = STK11XX_AVG2( \
						rgb[(i*width+1)*3 + 2], \
						rgb[((i+1)*width+1)*3 + 2] \
				);
			} else { // i&1 == 1
				// calc red , 2x top , 1xtopright, 1xright
				rgb[(i*width)*3] = STK11XX_AVG4( \
						rgb[((i-1)*width)*3], \
						rgb[((i-1)*width)*3], \
						rgb[((i-1)*width+1)*3 ], \
						rgb[(i*width+1)*3] \
				);
				// calc blue , avg top and right
				rgb[(i*width)*3+2] = STK11XX_AVG2( \
						rgb[((i-1)*width)*3 + 2], \
						rgb[(i*width+1)*3 + 2] \
				);
			}
			//right border
			if( (i&1) == 0) {
				// calc red , take left and bottom left
				rgb[lastcolumn + (i*width)*3] = STK11XX_AVG2( \
						rgb[lastcolumn + (i*width - 1)*3], \
						rgb[lastcolumn + ((i+1)*width - 1)*3] \
				);
				// calc blue , left and bottom
				rgb[lastcolumn + (i*width)*3+2] = STK11XX_AVG2( \
						rgb[lastcolumn + (i*width - 1)*3+2], \
						rgb[lastcolumn + ((i+1)*width)*3+2] \
				);
			} else { // i&1 == 1
				// calc red , top and left
				rgb[lastcolumn + (i*width)*3] = STK11XX_AVG2( \
						rgb[lastcolumn + ((i-1)*width - 1)*3], \
						rgb[lastcolumn + (i*width - 1)*3] \
				);
				// calc green , 2xtop , topleft, left
				rgb[lastcolumn + (i*width)*3 + 1] = STK11XX_AVG4( \
						rgb[lastcolumn + ((i-1)*width)*3 + 1], \
						rgb[lastcolumn + ((i-1)*width)*3 + 1], \
						rgb[lastcolumn + ((i-1)*width - 1)*3 + 1], \
						rgb[lastcolumn + (i*width - 1)*3 + 1] \
				);
			}
			
		}
	
	}


} 


/** 
 * @param in Buffer to RGB data
 * @param width Width of frame
 * @param height Height of frame
 *
 * @retval out Buffer to RGB data (resized)
 *
 * @brief Resize an image.
 *
 * This function permits to resize a decompressed image from a factor.
 */
void stk11xx_resize_image(uint8_t *out, uint8_t *in,
		const int width, const int height, const int factor)
{
	int x, y;

	int nwidth;
	int nheight;

	nheight = height / factor;
	nwidth = width / factor;

	for (y=0; y<nheight; y++) {
		for (x=0; x<nwidth; x++) {
			// R
			out[y * nwidth * 3 + x*3 + 0] = in[y*factor * width * 3 + x*factor*3 + 0];
			// G
			out[y * nwidth * 3 + x*3 + 1] = in[y*factor * width * 3 + x*factor*3 + 1];
			// B
			out[y * nwidth * 3 + x*3 + 2] = in[y*factor * width * 3 + x*factor*3 + 2];
		}
	}
}


/** 
 * @param rgb Buffer to RGB data
 * @param width Width of frame
 * @param height Height of frame
 * @param brightness Brightness correction
 *
 * @retval rgb Buffer to RGB data
 *
 * @brief Correct the brightness of an image.
 *
 * This function permits to correct the brightness of an image.
 */
void stk11xx_correct_brightness(uint8_t *rgb, const int width, const int height, const int brightness)
{
	int i;
	int x;

	if (brightness >= 32767) {
		x = (brightness - 32767) / 256;

		for (i = 0; i < (width * height * 3); i++) {
			if ((*(rgb + i) + (unsigned char) x) > 255)
				*(rgb + i) = 255;
			else
				*(rgb + i) += (unsigned char) x;
		}
	}
	else {
		x = (32767 - brightness) / 256;

		for (i = 0; i < (width * height * 3); i++) {
			if ((unsigned char) x > *(rgb + i))
				*(rgb + i) = 0;
			else
				*(rgb + i) -= (unsigned char) x;
		}
	}
}

